package com.hzya.frame.wecom.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.sysnew.application.entity.SysExtensionApiEntity;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.wecom.service.IWeComService;
import com.hzya.frame.wecom.util.WeComAccessToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

/**
 * @Description 企业微信service
 * @Author xiangerlin
 * @Date 2024/9/23 14:24
 **/
@Service(value = "weComServiceImpl")
public class WeComServiceImpl implements IWeComService {
    static Logger logger = LoggerFactory.getLogger(WeComServiceImpl.class);
    /**
     * 获取accessToken
     * https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=ID&corpsecret=SECRET
     * @param json
     * @return
     */
    @Override
    public JSONObject accessToken(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (null == jsonObject){
            return this.error("参数不能为空");
        }
        String corpid = jsonObject.getString("corpid");
        String corpsecret = jsonObject.getString("corpsecret");
        if (StrUtil.isEmpty(corpid)) {
            return this.error("corpid不能为空");
        }
        if (StrUtil.isEmpty(corpsecret)) {
            return this.error("corpsecret不能为空");
        }
        try {
            String accessToken = WeComAccessToken.getAccessToken(corpid, corpsecret);
            return this.ok(accessToken);
        }catch(Exception e){
            logger.error("获取accessToken出错",e);
        }
        return this.error("系统异常");
    }

    /**
     * 发送消息
     * https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token=ACCESS_TOKEN
     * @param json
     * @return
     */
    @Override
    public JSONObject messageSend(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (null == jsonObject){
            return this.error("参数不能为空");
        }
        String agentid = jsonObject.getString("agentid");
        String text = jsonObject.getString("text");
        String access_token = jsonObject.getString("access_token");
        if (StrUtil.isEmpty(agentid)){
            return this.error("agentid不能为空");
        }
        if (StrUtil.isEmpty(text)){
            return this.error("消息内容不能为空");
        }
        if (StrUtil.isEmpty(access_token)){
            return this.error("access_token不能为空");
        }
        String url = "https://qyapi.weixin.qq.com/cgi-bin/message/send?access_token="+access_token;
        jsonObject.remove("access_token");
        String param = jsonObject.toString();
        String res = HttpRequest.post(url).body(param).timeout(30000).execute().body();
        if (StrUtil.isNotEmpty(res)){
            JSONObject msgResponse = JSONObject.parseObject(res);
            String errcode = msgResponse.getString("errcode");
            String errmsg = msgResponse.getString("errmsg");
            if ("0".equals(errcode)){
                return ok();
            }else {
                return error(errmsg);
            }
        }
        return this.error("操作失败");
    }

    /**
     * 根据授权码获取用户信息
     * 单点登录的时候用
     * https://qyapi.weixin.qq.com/cgi-bin/auth/getuserinfo?access_token=ACCESS_TOKEN&code=CODE
     *
     * @param json
     * @return
     */
    @Override
    public JSONObject getUserInfoByAuthCode(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (null == jsonObject){
            return this.error("参数不能为空");
        }
        String access_token = jsonObject.getString("access_token");
        String code = jsonObject.getString("code");
        if (StrUtil.isEmpty(access_token)){
            //如果token为空，手动获取一次
            String corpid = jsonObject.getString("corpid");
            String corpsecret = jsonObject.getString("corpsecret");
            if (StrUtil.isEmpty(corpid)){
                this.error("corpid不能为空");
            }
            if (StrUtil.isEmpty(corpsecret)){
                this.error("corpsecret不能为空");
            }
            access_token = WeComAccessToken.getAccessToken(corpid, corpsecret);
        }
        if (StrUtil.isEmpty(code)){
            return  error("code不能为空");
        }
        String url = "https://qyapi.weixin.qq.com/cgi-bin/auth/getuserinfo?access_token="+access_token+"&code="+code;
        String res = HttpRequest.get(url).timeout(30000).execute().body();
        if (StrUtil.isNotEmpty(res)){
            JSONObject msgResponse = JSONObject.parseObject(res);
            String errcode = msgResponse.getString("errcode");
            String errmsg = msgResponse.getString("errmsg");
            String userid = msgResponse.getString("userid");
            if ("0".equals(errcode)){
                return ok(userid);
            }else {
                return error(errmsg);
            }
        }
        return null;
    }

    /**
     * 根据userid读取成员信息
     * https://qyapi.weixin.qq.com/cgi-bin/user/get?access_token=ACCESS_TOKEN&userid=USERID
     *
     * @param json
     * @return
     */
    @Override
    public JSONObject getUserInfoByUserId(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (null == jsonObject){
            return this.error("参数不能为空");
        }
        String access_token = jsonObject.getString("access_token");
        String userid = jsonObject.getString("userid");
        if (StrUtil.isEmpty(access_token)){
            return  error("access_token不能为空");
        }
        if (StrUtil.isEmpty(userid)){
            return  error("userid不能为空");
        }
        String url = "https://qyapi.weixin.qq.com/cgi-bin/user/get?access_token="+access_token+"&userid="+userid;
        String res = HttpRequest.get(url).timeout(30000).execute().body();
        if (StrUtil.isNotEmpty(res)){
            JSONObject msgResponse = JSONObject.parseObject(res);
            String errcode = msgResponse.getString("errcode");
            String errmsg = msgResponse.getString("errmsg");
            if ("0".equals(errcode)){
                return ok(res);
            }else {
                return error(errmsg);
            }
        }
        return null;
    }

    /**
     * 根据手机号获取userid
     * https://qyapi.weixin.qq.com/cgi-bin/user/getuserid?access_token=ACCESS_TOKEN
     *
     * @param json
     * @return
     */
    @Override
    public JSONObject getUserIdByMobile(JSONObject json) {
        JSONObject jsonObject = json.getJSONObject("jsonStr");
        if (null == jsonObject){
            return this.error("参数不能为空");
        }
        String access_token = jsonObject.getString("access_token");
        String mobile = jsonObject.getString("mobile");
        if (StrUtil.isEmpty(access_token)){
            return  error("access_token不能为空");
        }
        if (StrUtil.isEmpty(mobile)){
            return  error("mobile不能为空");
        }
        String url = "https://qyapi.weixin.qq.com/cgi-bin/user/getuserid?access_token="+access_token;
        jsonObject.remove("access_token");
        String param = jsonObject.toString();
        String res = HttpRequest.post(url).body(param).timeout(30000).execute().body();
        if (StrUtil.isNotEmpty(res)){
            JSONObject msgResponse = JSONObject.parseObject(res);
            String errcode = msgResponse.getString("errcode");
            String errmsg = msgResponse.getString("errmsg");
            String userid = msgResponse.getString("userid");
            if ("0".equals(errcode)){
                return ok(userid);
            }else {
                return error(errmsg);
            }
        }
        return null;
    }

    /**
     * 成功
     * @return
     */
    private static JSONObject ok(){
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code","200");
        jsonObject.put("msg","成功");
        jsonObject.put("data","");
        return jsonObject;
    }

    /**
     * 成功
     * @param data 返回数据
     * @return
     */
    private static JSONObject ok(String data){
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code","200");
        jsonObject.put("msg","成功");
        jsonObject.put("data",data);
        return jsonObject;
    }

    /**
     * 失败
     * @param msg 失败原因
     * @return
     */
    private static JSONObject error(String msg){
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("code","500");
        jsonObject.put("msg",msg);
        jsonObject.put("data","");
        return jsonObject;
    }
}
